#!/bin/bash

TIMEOUT=1000
MEMLIMIT=8000000

GENERATE_CHCS=yes

CHECK_INLINED=yes
CHECK_COMPOSITE=yes
CHECK_COMPOSITE_MA=yes

# Sets the encoding that the exported CHCs will use
# SBE = Single Block Encoding
# BBE = Basic Block Encoding
# LBE = Large Block Encoding 
#
# Note that although LBE is known to perform best for SMT-based verification it
# will yield rather complex CHCs. When trying to understand the CHCs, try using
# SBE as this will yield simpler CHCs and CFAs.
BLOCK_ENCODING=LBE

###############################################################################
# Auxiliary functions
###############################################################################
function parseSpec {
    local file
    local main
    local formula
    
    IFS=":"
    while read -r name value
    do
        case $name in 
        "file")
            file="$value"
            ;;
        "main")
            main="$value"
            ;;
        "formula")
            formula="$value"
            ;;
        esac
    done < $1
    echo "$file:$main:$formula"
}

# Go to the one this file is contained in
SCRIPTPATH=$(dirname "$0")
cd $SCRIPTPATH

###############################################################################
# Generate CHCs from given specs for ST programs
###############################################################################

if [ "$GENERATE_CHCS" = "yes" ]; then
    echo "Generating CHCs from given specs ..."
    for fn in programs/st/*.spec; do
        IFS=":" read file main formula<<<"$(parseSpec $fn)"
        echo -n " $fn ..."
        scripts/run_arcade.sh $file $main "$formula" $BLOCK_ENCODING > output/arcade/`basename $fn`.log 2>&1
        echo "DONE"
    done
else
    echo "Skipping generation of CHCs from given specs"
fi

###############################################################################
# Z3
###############################################################################

if [ "$CHECK_INLINED" = "yes" ]; then
    echo "Checking CHCs of programs with inlined calls ..."
        >output/inlined_time.txt
        for fn in programs/smtlib/inlined/*.smt2; do
        echo -n " $fn ... "
        echo "Checking " `basename $fn` ":">>output/inlined_time.txt
        (time (ulimit -t $TIMEOUT -v $MEMLIMIT; scripts/run_z3.sh $fn > output/inlined/`basename $fn`.log 2>&1) 2>>output/inlined_time.txt)
        echo "============================">>output/inlined_time.txt
        echo "DONE"
    done
else
    echo "Skipping CHC-solving for programs with inlined calls"
fi

if [ "$CHECK_COMPOSITE" = "yes" ]; then
    echo "Checking CHCs of programs with calls ..."
        >output/composite_time.txt
        for fn in programs/smtlib/composite/*.smt2; do
        echo -n " $fn ... "
        echo "Checking " `basename $fn` ":">>output/composite_time.txt
        (time (ulimit -t $TIMEOUT -v $MEMLIMIT; scripts/run_z3.sh $fn > output/composite/`basename $fn`.log 2>&1) 2>>output/composite_time.txt)
        echo "============================">>output/composite_time.txt
        echo "DONE"
    done
else
    echo "Skipping CHC-solving for programs with calls"
fi

if [ "$CHECK_COMPOSITE_MA" = "yes" ]; then
    echo "Checking CHCs of programs with calls and mode abstraction..."
        >output/composite_ma_time.txt
        for fn in programs/smtlib/composite_ma/*.smt2; do
        echo -n " $fn ... "
        echo "Checking " `basename $fn` ":">>output/composite_ma_time.txt
        (time (ulimit -t $TIMEOUT -v $MEMLIMIT; scripts/run_z3.sh $fn > output/composite_ma/`basename $fn`.log 2>&1) 2>>output/composite_ma_time.txt)
        echo "============================">>output/composite_ma_time.txt
        echo "DONE"
    done
else
    echo "Skipping CHC-solving for programs with calls and mode abstraction"
fi
